@@ -3,14 +3,15 @@ from typing_extensions import Literal
 from enum import Enum
 
 from pathlib import Path
-import time
+from urllib.parse import urlparse, quote_plus
+from base64 import standard_b64encode
 import re
 import json
-from urllib.parse import urlparse, quote_plus
 
 import requests
 from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
 
+from baidupcs_py.common.date import now_timestamp
 from baidupcs_py.common.io import RangeRequestIO, DEFAULT_MAX_CHUNK_SIZE
 from baidupcs_py.common.cache import timeout_cache
 from baidupcs_py.common.crypto import calu_md5, calu_sha1
@@ -48,10 +49,18 @@ class Method(Enum):
 
 
 class PcsNode(Enum):
+    """Pan Nodes which use pcs.baidu.com"""
+
     Quota = "rest/2.0/pcs/quota"
     File = "rest/2.0/pcs/file"
-    PcsCloud = "rest/2.0/pcs/services/cloud_dl"
-    PanCloud = "rest/2.0/services/cloud_dl"
+
+    def url(self) -> str:
+        return f"{PCS_BAIDU_COM}/{self.value}"
+
+
+class PanNode(Enum):
+    """Pan Nodes which use pan.baidu.com"""
+
     TransferShared = "share/transfer"
     Share = "share/set"
     SharedPathList = "share/list"
@@ -59,8 +68,12 @@ class PcsNode(Enum):
     SharedCancel = "share/cancel"
     SharedPassword = "share/surlinfoinrecord"
     Getcaptcha = "api/getcaptcha"
+    Cloud = "rest/2.0/services/cloud_dl"
     UserProducts = "rest/2.0/membership/user"
 
+    def url(self) -> str:
+        return f"{PAN_BAIDU_COM}/{self.value}"
+
 
 class BaiduPCS:
     """`BaiduPCS` provides pcs's apis which return raw json"""
@@ -88,7 +101,14 @@ class BaiduPCS:
         self._bduss = bduss
         self._stoken = stoken
         self._ptoken = ptoken
-        self._bdstoken = self.bdstoken()
+        self._bdstoken = calu_md5(bduss)
+        self._logid = None
+        self._baiduid = cookies.get("BAIDUID")
+        if self._baiduid:
+            self._logid = standard_b64encode(self._baiduid.encode("ascii")).decode(
+                "utf-8"
+            )
+
         self._cookies = cookies
         self._session = requests.Session()
         self._session.cookies.update(cookies)
@@ -106,10 +126,6 @@ class BaiduPCS:
     def cookies(self) -> Dict[str, Optional[str]]:
         return self._session.cookies.get_dict()
 
-    @staticmethod
-    def _form_url(node: PcsNode, domain: str = PCS_BAIDU_COM):
-        return f"{domain}/{node.value}"
-
     @staticmethod
     def _app_id(url: str):
         """Select app_id based on `url`"""
@@ -175,15 +191,11 @@ class BaiduPCS:
     ) -> requests.Response:
         return self._request(Method.Get, url, params=params, headers=headers)
 
-    def bdstoken(self) -> Optional[str]:
-        bdstoken = calu_md5(self._bduss)
-        return bdstoken
-
     @assert_ok
     def quota(self):
         """Quota space information"""
 
-        url = self._form_url(PcsNode.Quota)
+        url = PcsNode.Quota.url()
         params = {"method": "info"}
         resp = self._request(Method.Get, url, params=params)
         return resp.json()
@@ -230,7 +242,7 @@ class BaiduPCS:
         time: bool = False,
         size: bool = False,
     ):
-        url = self._form_url(PcsNode.File)
+        url = PcsNode.File.url()
         orderby = None
         if name:
             orderby = "name"
@@ -259,10 +271,15 @@ class BaiduPCS:
         ondup="overwrite",
         callback: Callable[[MultipartEncoderMonitor], None] = None,
     ):
+        """Upload the content of io to remotepath
+
+        WARNING: This api can not set local_ctime and local_mtime
+        """
+
         assert remotepath.startswith("/"), "`remotepath` must be an absolute path"
         remotePath = Path(remotepath)
 
-        url = self._form_url(PcsNode.File)
+        url = PcsNode.File.url()
         params = {
             "method": "upload",
             "ondup": ondup,
@@ -285,6 +302,8 @@ class BaiduPCS:
         content_crc32: int,  # not needed
         io_len: int,
         remotepath: str,
+        local_ctime: Optional[int] = None,
+        local_mtime: Optional[int] = None,
         ondup="overwrite",
     ):
         """Rapid Upload File
@@ -299,17 +318,22 @@ class BaiduPCS:
 
         assert remotepath.startswith("/"), "`remotepath` must be an absolute path"
 
-        url = self._form_url(PcsNode.File)
+        url = PcsNode.File.url()
         params = {
             "method": "rapidupload",
             "BDUSS": self._bduss,
         }
+
+        ntp = now_timestamp()
+
         data = {
             "path": remotepath,
             "content-length": io_len,
             "content-md5": content_md5,
             "slice-md5": slice_md5,
             "content-crc32": content_crc32,
+            "local_ctime": str(local_ctime or ntp),
+            "local_mtime": str(local_mtime or ntp),
             "ondup": ondup,
         }
 
@@ -324,7 +348,7 @@ class BaiduPCS:
     def upload_slice(
         self, io: IO, callback: Callable[[MultipartEncoderMonitor], None] = None
     ):
-        url = self._form_url(PcsNode.File)
+        url = PcsNode.File.url()
         params = {
             "method": "upload",
             "type": "tmpfile",
@@ -338,21 +362,35 @@ class BaiduPCS:
         return resp.json()
 
     @assert_ok
-    def combine_slices(self, slice_md5s: List[str], remotepath: str, ondup="overwrite"):
-        url = self._form_url(PcsNode.File)
+    def combine_slices(
+        self,
+        slice_md5s: List[str],
+        remotepath: str,
+        local_ctime: Optional[int] = None,
+        local_mtime: Optional[int] = None,
+        ondup="overwrite",
+    ):
+        url = PcsNode.File.url()
         params = {
             "method": "createsuperfile",
             "path": remotepath,
             "ondup": ondup,
             "BDUSS": self._bduss,
         }
-        data = {"param": dump_json({"block_list": slice_md5s})}
+
+        ntp = now_timestamp()
+
+        data = {
+            "param": dump_json({"block_list": slice_md5s}),
+            "local_ctime": str(local_ctime or ntp),
+            "local_mtime": str(local_mtime or ntp),
+        }
         resp = self._request(Method.Post, url, params=params, data=data)
         return resp.json()
 
     @assert_ok
     def search(self, keyword: str, remotepath: str, recursive: bool = False):
-        url = self._form_url(PcsNode.File)
+        url = PcsNode.File.url()
         params = {
             "method": "search",
             "path": remotepath,
@@ -364,7 +402,7 @@ class BaiduPCS:
 
     @assert_ok
     def makedir(self, directory: str):
-        url = self._form_url(PcsNode.File)
+        url = PcsNode.File.url()
         params = {
             "method": "mkdir",
             "path": directory,
@@ -373,7 +411,7 @@ class BaiduPCS:
         return resp.json()
 
     def file_operate(self, operate: str, param: List[Dict[str, str]]):
-        url = self._form_url(PcsNode.File)
+        url = PcsNode.File.url()
         params = {"method": operate}
         data = {"param": dump_json({"list": param})}
         resp = self._request(Method.Post, url, params=params, data=data)
@@ -457,7 +495,7 @@ class BaiduPCS:
 
     @assert_ok
     def cloud_operate(self, params: Dict[str, str]):
-        url = self._form_url(PcsNode.PanCloud, domain=PAN_BAIDU_COM)
+        url = PanNode.Cloud.url()
         resp = self._request(Method.Get, url, params=params)
         return resp.json()
 
@@ -528,12 +566,12 @@ class BaiduPCS:
         meta = self.meta(*remotepaths)
         fs_ids = [i["fs_id"] for i in meta["list"]]
 
-        url = self._form_url(PcsNode.Share, domain=PAN_BAIDU_COM)
+        url = PanNode.Share.url()
         params = {
             "channel": "chunlei",
             "clienttype": "0",
             "web": "1",
-            "bdstoken": self.bdstoken() or "",
+            "bdstoken": self._bdstoken,
         }
         data = {
             "fid_list": dump_json(fs_ids),
@@ -560,7 +598,7 @@ class BaiduPCS:
             - 4, with password
         """
 
-        url = self._form_url(PcsNode.SharedRecord, domain=PAN_BAIDU_COM)
+        url = PanNode.SharedRecord.url()
         params = {
             "page": str(page),
             "desc": "1",
@@ -575,7 +613,7 @@ class BaiduPCS:
         Only return password
         """
 
-        url = self._form_url(PcsNode.SharedPassword, domain=PAN_BAIDU_COM)
+        url = PanNode.SharedPassword.url()
         params = {
             "shareid": str(share_id),
             "sign": calu_md5(f"{share_id}_sharesurlinfo!@#"),
@@ -585,7 +623,7 @@ class BaiduPCS:
 
     @assert_ok
     def cancel_shared(self, *share_ids: int):
-        url = self._form_url(PcsNode.SharedCancel, domain=PAN_BAIDU_COM)
+        url = PanNode.SharedCancel.url()
         data = {
             "shareid_list": dump_json(share_ids),
         }
@@ -606,7 +644,7 @@ class BaiduPCS:
         init_url = self.shared_init_url(shared_url)
         params = {
             "surl": init_url.split("surl=")[-1],
-            "t": str(int(time.time() * 1000)),
+            "t": str(now_timestamp() * 1000),
             "channel": "chunlei",
             "web": "1",
             "bdstoken": "null",
@@ -627,7 +665,7 @@ class BaiduPCS:
 
     @assert_ok
     def getcaptcha(self, shared_url: str) -> str:
-        url = self._form_url(PcsNode.Getcaptcha, domain=PAN_BAIDU_COM)
+        url = PanNode.Getcaptcha.url()
         params = {
             "prod": "shareverify",
             "channel": "chunlei",
@@ -670,19 +708,19 @@ class BaiduPCS:
     def list_shared_paths(self, sharedpath: str, uk: int, share_id: int):
         assert self._stoken, "`STOKEN` is not in `cookies`"
 
-        url = self._form_url(PcsNode.SharedPathList, domain=PAN_BAIDU_COM)
+        url = PanNode.SharedPathList.url()
         params = {
             "channel": "chunlei",
             "clienttype": "0",
             "web": "1",
             "num": "10000",
             "dir": sharedpath,
-            "t": str(int(time.time() * 1000)),
+            "t": str(now_timestamp() * 1000),
             "uk": str(uk),
             "shareid": str(share_id),
             # 'desc': 1,   ## reversely
             "order": "name",  # sort by name, or size, time
-            "_": str(int(time.time() * 1000)),
+            "_": str(now_timestamp() * 1000),
             # 'bdstoken': self._get_bdstoken()
         }
         resp = self._request(Method.Get, url, params=params)
@@ -700,7 +738,7 @@ class BaiduPCS:
     ):
         """`remotedir` must exist"""
 
-        url = self._form_url(PcsNode.TransferShared, domain=PAN_BAIDU_COM)
+        url = PanNode.TransferShared.url()
         params = {
             "shareid": str(share_id),
             "from": str(uk),
@@ -725,7 +763,7 @@ class BaiduPCS:
     @assert_ok
     def user_info(self):
         bduss = self._bduss
-        timestamp = str(int(time.time()))
+        timestamp = str(now_timestamp())
         model = get_phone_model(bduss)
         phoneIMEIStr = sum_IMEI(bduss)
 
@@ -783,7 +821,7 @@ class BaiduPCS:
             "Cookie": "ka=open",
             "net": "1",
             "User-Agent": "bdtb for Android 6.9.2.1",
-            "client_logid": str(int(time.time() * 1000)),
+            "client_logid": str(now_timestamp() * 1000),
             "Connection": "Keep-Alive",
         }
         resp = requests.get(url, headers=headers, params=None)
@@ -791,7 +829,7 @@ class BaiduPCS:
 
     @assert_ok
     def user_products(self):
-        url = self._form_url(PcsNode.UserProducts, domain=PAN_BAIDU_COM)
+        url = PanNode.UserProducts.url()
         params = {
             "method": "query",
         }
@@ -818,7 +856,7 @@ class BaiduPCS:
         bduss = self._bduss
         uid = str(self._user_id) or ""
 
-        timestamp = str(int(time.time() * 1000))
+        timestamp = str(now_timestamp() * 1000)
         devuid = "0|" + calu_md5(bduss).upper()
 
         enc = calu_sha1(bduss)
@@ -826,7 +864,7 @@ class BaiduPCS:
             enc + uid + "ebrcUYiuxaZv2XGu7KIYKxUrqfnOfpDF" + timestamp + devuid
         )
 
-        url = self._form_url(PcsNode.File, domain=PCS_BAIDU_COM)
+        url = PcsNode.File.url()
         params = {
             "method": "locatedownload",
             "ver": "2",
@@ -871,7 +909,7 @@ class BaiduPCS:
     def m3u8_stream(self, remotepath: str, type: M3u8Type = "M3U8_AUTO_720"):
         """Get content of the m3u8 stream file"""
 
-        url = self._form_url(PcsNode.File, domain=PCS_BAIDU_COM)
+        url = PcsNode.File.url()
         params = {
             "method": "streaming",
             "path": remotepath,
